@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2016 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -56,7 +56,12 @@ class SpamDetected(Errors.DiscardMessage):
     """The message contains known spam"""
 
 class HeaderMatchHold(Errors.HoldMessage):
-    reason = _('The message headers matched a filter rule')
+    def __init__(self, pattern):
+        self.__pattern = pattern
+
+    def reason_notice(self):
+        pattern = self.__pattern
+        return _('Header matched regexp: %(pattern)s')
 
 
 # And reset the translator
@@ -104,7 +109,8 @@ def process(mlist, msg, msgdata):
         msgdata['from_is_list'] = 0
         dn, addr = parseaddr(msg.get('from'))
         if addr and mlist.dmarc_moderation_action > 0:
-            if Utils.IsDMARCProhibited(mlist, addr):
+            if (mlist.GetPattern(addr, mlist.dmarc_moderation_addresses) or
+                Utils.IsDMARCProhibited(mlist, addr)):
                 # Note that for dmarc_moderation_action, 0 = Accept, 
                 #    1 = Munge, 2 = Wrap, 3 = Reject, 4 = Discard
                 if mlist.dmarc_moderation_action == 1:
@@ -195,6 +201,7 @@ error, contact the mailing list owner at %(listowner)s."""))
                         # pass it here but list-owner can set this to be
                         # discarded on the GUI if he wants.
                         return
-                    hold_for_approval(mlist, msg, msgdata, HeaderMatchHold)
+                    hold_for_approval(
+                        mlist, msg, msgdata, HeaderMatchHold(pattern))
                 if action == mm_cfg.ACCEPT:
                     return
